/**
 * @file matrix.c
 * @author Florian Burgener (florian.burgener@etu.hesge.ch)
 * @brief Implémentation de librairie matrix.
 * @version 1.0
 * @date 2021-11-16
 *
 * @copyright Copyright (c) 2021
 *
 */

#include "matrix.h"

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

error_code matrix_alloc(matrix *mat, int32_t m, int32_t n) {
    mat->m = m;
    mat->n = n;
    int32_t **data = (int32_t **)malloc(sizeof(int32_t *) * mat->m);

    if (data == NULL) {
        return err;
    }

    mat->data = data;

    for (int32_t i = 0; i < m; i += 1) {
        int32_t *row = (int32_t *)malloc(sizeof(int32_t) * n);

        if (row == NULL) {
            return err;
        }

        mat->data[i] = row;
    }

    return ok;
}

error_code matrix_init(matrix *mat, int32_t m, int32_t n, int32_t val) {
    if (matrix_alloc(mat, m, n) == err) {
        return err;
    }

    for (int32_t y = 0; y < m; y += 1) {
        for (int32_t x = 0; x < n; x += 1) {
            mat->data[y][x] = val;
        }
    }

    return ok;
}

error_code matrix_print(const matrix mat) {
    for (int32_t y = 0; y < mat.m; y += 1) {
        for (int32_t x = 0; x < mat.n; x += 1) {
            printf("%d, ", mat.data[y][x]);
        }

        printf("\n");
    }

    return ok;
}

error_code matrix_destroy(matrix *mat) {
    for (int32_t y = 0; y < mat->m; y += 1) {
        free(mat->data[y]);
    }

    free(mat->data);
    mat->m = -1;
    mat->n = -1;
    mat->data = NULL;

    return ok;
}

error_code matrix_init_from_array(matrix *mat, int32_t m, int32_t n, int32_t data[], int32_t s) {
    if (m * n != s) {
        return err;
    }

    if (matrix_alloc(mat, m, n) == err) {
        return err;
    }

    for (int32_t y = 0; y < mat->m; y += 1) {
        for (int32_t x = 0; x < mat->n; x += 1) {
            mat->data[y][x] = data[y * mat->n + x];
        }
    }

    return ok;
}

error_code matrix_clone(matrix *cloned, const matrix mat) {
    if (matrix_alloc(cloned, mat.m, mat.n) == err) {
        return err;
    }

    for (int32_t y = 0; y < mat.m; y += 1) {
        for (int32_t x = 0; x < mat.n; x += 1) {
            cloned->data[y][x] = mat.data[y][x];
        }
    }

    return ok;
}

error_code matrix_transpose(matrix *transposed, const matrix mat) {
    if (matrix_alloc(transposed, mat.n, mat.m) == err) {
        return err;
    }

    for (int32_t y = 0; y < mat.m; y += 1) {
        for (int32_t x = 0; x < mat.n; x += 1) {
            transposed->data[x][y] = mat.data[y][x];
        }
    }

    return ok;
}

error_code matrix_extract_submatrix(matrix *sub, const matrix mat, int32_t m0, int32_t m1, int32_t n0, int32_t n1) {
    if (m1 <= m0 || n1 <= n0 || m0 < 0 || n0 < 0 || m1 > mat.m || n1 > mat.n) {
        return err;
    }

    if (matrix_alloc(sub, m1 - m0, n1 - n0) == err) {
        return err;
    }

    for (int32_t y = m0; y < m1; y += 1) {
        for (int32_t x = n0; x < n1; x += 1) {
            sub->data[y - m0][x - n0] = mat.data[y][x];
        }
    }

    return ok;
}

bool matrix_is_equal(matrix mat1, matrix mat2) {
    if (mat1.m != mat2.m || mat1.n != mat2.n) {
        return false;
    }

    for (int32_t y = 0; y < mat1.m; y += 1) {
        for (int32_t x = 0; x < mat2.n; x += 1) {
            if (mat1.data[y][x] != mat2.data[y][x]) {
                return false;
            }
        }
    }

    return true;
}

error_code matrix_get(int32_t *elem, const matrix mat, int32_t ix, int32_t iy) {
    if (ix < 0 || ix >= mat.n || iy < 0 || iy >= mat.m) {
        return err;
    }

    *elem = mat.data[iy][ix];
    return ok;
}

error_code matrix_set(matrix mat, int32_t ix, int32_t iy, int32_t elem) {
    if (ix < 0 || ix >= mat.n || iy < 0 || iy >= mat.m) {
        return err;
    }

    mat.data[iy][ix] = elem;
    return ok;
}
